@@ -11,8 +11,6 @@ use Data::Dumper ;
 use Data::TreeDumper ;
 use Time::HiRes ;
 use Tie::Hash::Indexed ;
-use File::Basename ;
-use File::Spec::Functions qw(:ALL) ;
 
 require Exporter ;
 use AutoLoader qw(AUTOLOAD) ;
@@ -21,7 +19,7 @@ our @ISA = qw(Exporter) ;
 our %EXPORT_TAGS = ('all' => [ qw() ]) ;
 our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } ) ;
 our @EXPORT = qw(CreateDependencyTree) ;
-our $VERSION = '0.08' ;
+our $VERSION = '0.06' ;
 
 use PBS::PBS ;
 use PBS::Output ;
@@ -30,101 +28,13 @@ use PBS::Triggers ;
 use PBS::PostBuild ;
 use PBS::Plugin;
 
-#-----------------------------------------------------------------------------------------
-
-sub SYNCHRONIZE
-{
-die "SYNCHRONIZE expected 3 arguments" unless 3 == @_ ;
-
-my
-(
-  $unsynchronized_dependency_file_name
-, $dependency_file_name
-, $message_format
-) = @_ ;
-
-return
-	(
-	bless 
-		(
-		{
-		  SOURCE_FILE      => $unsynchronized_dependency_file_name
-		, DESTINATION_FILE => $dependency_file_name
-		, MESSAGE_FORMAT   => $message_format
-		}
-		, "PBS_SYNCHRONIZE"
-		)
-	) ;
-}
-
-sub FORCE_TRIGGER
-{
-my $reason = shift || die "Forced trigger must have a reason\n" ;
-
-return
-	(
-	bless
-		(
-		  { MESSAGE => $reason }
-		, "PBS_FORCE_TRIGGER"
-		)
-	) ;
-}
-
-#-----------------------------------------------------------------------------------------
-
-sub SynchronizeAfterBuild
-{
-# called everytime a node is build.
-# check if any of the node dependencies needs synchronization
-
-my $node = shift ;
-
-for my $dependency_name (keys %$node)
-	{
-	next if $dependency_name =~ /^__/ ; # eliminate private data
-	
-	if(exists $node->{$dependency_name}{__SYNCHRONIZE})
-		{
-		my $dependents_built = $node->{$dependency_name}{__SYNCHRONIZE_DEPENDENTS_BUILD} ;
-		$dependents_built = defined $dependents_built ? $dependents_built : 0 ;
-		$dependents_built += 1 ; # this node
-		
-		my $number_of_dependents = 1 ;
-		if(exists $node->{$dependency_name}{__LINKED})
-			{
-			$number_of_dependents += $node->{$dependency_name}{__LINKED} ;
-			}
-			
-		if($dependents_built == $number_of_dependents)
-			{
-			my $synch_list = $node->{$dependency_name}{__SYNCHRONIZE} ;
-			for my $file_to_synchronize (keys %$synch_list)
-				{
-				my $message_format = $synch_list->{$file_to_synchronize}{MESSAGE_FORMAT} ;
-				PrintWarning sprintf($message_format , $dependency_name) unless $PBS::Shell::silent_commands ;
-				
-				unless(rename($file_to_synchronize, $synch_list->{$file_to_synchronize}{TO_FILE}))
-					{
-					PrintError "Error synchronizing '$file_to_synchronize' to '$synch_list->{$file_to_synchronize}{TO_FILE}': $!\n" ;
-					}
-				}
-			}
-		else	
-			{
-			$node->{$dependency_name}{__SYNCHRONIZE_DEPENDENTS_BUILD} = $dependents_built ;
-			}
-		}
-	}
-}
-
 #-------------------------------------------------------------------------------
 
 sub CreateDependencyTree
 {
 my $Pbsfile          = shift ;
 my $package_alias    = shift ;
-my $load_package     = PBS::PBS::CanonizePackageName(shift) ;
+my $load_package     = shift ;
 my $pbs_config       = shift ;
 my $tree             = shift ;
 my $config           = shift ; 
@@ -147,9 +57,11 @@ for my $regex (@{$pbs_config->{DISPLAY_DEPENDENCIES_REGEX}})
 		}
 	}
 	
+$tree->{__DEPENDED}++ ; # depend sub tree once only flag
+$tree->{__DEPENDED_AT} = $Pbsfile ;
+
 my %dependency_rules ; # keep a list of  which rules generated which dependencies
 my $has_dependencies = 0 ;
-my $has_matching_non_subpbs_rules = 0 ;
 my @sub_pbs ; # list of subpbs matching this node
 
 tie my %triggered_nodes, 'Tie::Hash::Indexed';
@@ -211,7 +123,7 @@ for(my $rule_index = 0 ; $rule_index < @$dependency_rules ; $rule_index++)
 		$DB::single = 1 if(PBS::Debug::CheckBreakpoint(%debug_data, PRE => 1)) ;
 		}
 		
-	my ($dependency_result, $builder_override) = $depender->($node_name, $config, $tree, $inserted_nodes, $dependency_rules->[$rule_index]) ;
+	my ($dependency_result, $builder_override) = $depender->($node_name, $config, $tree, $inserted_nodes) ;
 	
 	my ($triggered, @dependencies ) = @$dependency_result ;
 	
@@ -220,9 +132,6 @@ for(my $rule_index = 0 ; $rule_index < @$dependency_rules ; $rule_index++)
 	
 	if($triggered)
 		{
-		$tree->{__DEPENDED}++ ; # depend sub tree once only flag
-		$tree->{__DEPENDED_AT} = $Pbsfile ;
-		
 		my $subs_list = $dependency_rules->[$rule_index]{NODE_SUBS} ;
 		
 		if(defined $subs_list)
@@ -267,50 +176,35 @@ for(my $rule_index = 0 ; $rule_index < @$dependency_rules ; $rule_index++)
 		if(@dependencies && 'HASH' eq ref $dependencies[0])
 			{
 			$dependencies[0]{__RULE_NAME} = $dependency_rules->[$rule_index]{NAME} ;
-			push @sub_pbs, 
-				{
-				  SUBPBS => $dependencies[0]
-				, RULE   => $dependency_rules->[$rule_index]
-				} ;
+			push @sub_pbs, $dependencies[0] ;
 			
 			if($pbs_config->{DEBUG_DISPLAY_DEPENDENCIES} && $node_name_matches_ddr)
 				{
-				PrintInfo("   $node_name has matching sub pbs, rule $rule_index:$rule_info\n") ;
+				PrintInfo("$node_name has matching sub pbs, rule $rule_index:$rule_info\n") ;
 				}
 				
 			next ;
 			}
-		else
-			{
-			$has_matching_non_subpbs_rules++ ;
-			}
-		
+			
 		#~ warn DumpTree(\@dependencies, "dependencies:") ;
 		
 		# transform the node name into an internal structure and check for node attributes
 		@dependencies = map
 				{
-				if(('' eq ref $_) && (! /^__/))
+				my ($dependency_name, $dependency_attribute) ;
+				
+				
+				if(/(.*)::(.*)$/)
 					{
-					if(/(.*)::(.*)$/)
+					# handle node user attribute
+					
+					# return a hash
 						{
-						# handle node user attribute
-						
-						# return a hash
-							{
-							  NAME => $1
-							, RULE_INDEX => $rule_index
-							, USER_ATTRIBUTE => $2
-							}
-						}
-					else
-						{
-						# return a hash
-							{
-							  NAME => $_
-							, RULE_INDEX => $rule_index
-							}
+						  NAME => $1
+						, RULE_INDEX => $rule_index
+						, USER_ATTRIBUTE => $2
 						}
+
 					}
 				else
 					{
@@ -358,35 +252,26 @@ for(my $rule_index = 0 ; $rule_index < @$dependency_rules ; $rule_index++)
 				$rule_type .= '[BO]' if($builder_override) ;
 				$rule_type .= '[S]'  if(defined $dependency_rules->[$rule_index]{NODE_SUBS}) ;
 				
-				my @dependency_names = map {$_->{NAME} ;} grep {'' eq ref $_->{NAME}} @dependencies ;
+				my @dependency_names = map {$_->{NAME} ;} @dependencies ;
 				
-				
-				my $forced_trigger = '' ;
-				if(grep {'PBS_FORCE_TRIGGER' eq ref $_->{NAME}} @dependencies) # use List::Utils::Any
+				my $dependency_info = "$node_name ${node_type}has dependencies [@dependency_names], rule $rule_index:$rule_info:$rule_type" ;
+				if(defined $tree->{__PBS_CONFIG}{DEBUG_DISPLAY_DEPENDENCY_REGEX})
 					{
-					$forced_trigger = 'FORCED_TRIGGER! ' ;
-					}
-					
-				my $dependency_info ;
-				if(defined $pbs_config->{DEBUG_DISPLAY_DEPENDENCIES_LONG})
-					{
-					$dependency_info = "'$node_name' ${node_type}${forced_trigger} rule $rule_index:$rule_info:$rule_type gives dependencies:\n"
-								. "      " . join("\n      ", map {"'$_'"} @dependency_names) ;
+					# the dependers will be indented, indent the result under them.
+					PrintInfo("      $dependency_info.\n") ;
 					}
 				else
 					{
-					$dependency_info = "'$node_name' ${node_type}${forced_trigger}has dependencies [@dependency_names], rule $rule_index:$rule_info:$rule_type" ;
+					PrintInfo("   $dependency_info.\n") ;
 					}
 					
-				PrintInfo("   $dependency_info\n") ;
-					
 				PrintWithContext
 					(
 					  $dependency_rules->[$rule_index]{FILE}
 					, 1, 2 #context  size
 					, $dependency_rules->[$rule_index]{LINE}
 					, \&INFO
-					) if defined $pbs_config->{DEBUG_DISPLAY_DEPENDENCY_RULE_DEFINITION} ;
+					) if defined $pbs_config->{DISPLAY_DEPENDENCY_RULE_DEFINITION} ;
 				}
 			}
 			
@@ -396,33 +281,15 @@ for(my $rule_index = 0 ; $rule_index < @$dependency_rules ; $rule_index++)
 		for my $dependency (@dependencies)
 			{
 			my $dependency_name = $dependency->{NAME} ;
-			if(ref $dependency_name eq 'PBS_FORCE_TRIGGER')
+			
+			if($dependency_name =~ /^__PBS_FORCE_TRIGGER/)
 				{
-				push @{$tree->{__PBS_FORCE_TRIGGER}}, $dependency_name ;
-				next ;
+				$tree->{$dependency_name} = $dependency ;
 				}
-				
-			if(ref $dependency_name eq 'PBS_SYNCHRONIZE')
-				{
-				my 
-				(
-				  $unsynchronized_dependency_file_name
-				, $dependency_file_name
-				, $message_format
-				) = @$dependency_name{'SOURCE_FILE', 'DESTINATION_FILE', 'MESSAGE_FORMAT'} ;
-				
-				$tree->{__SYNCHRONIZE}{$unsynchronized_dependency_file_name} = 
-					{
-					  TO_FILE        => $dependency_file_name
-					, MESSAGE_FORMAT => $message_format
-					} ;
-					
-				next ;
-				}
-				
+			
 			next if $dependency_name =~ /^__/ ;
 			
-			RunPluginSubs($pbs_config, 'CheckNodeName', $dependency_name, $dependency_rules->[$rule_index]) ;
+			RunPluginSubs('CheckNodeName', $dependency_name, $dependency_rules->[$rule_index]) ;
 			
 			if($node_name eq $dependency_name)
 				{
@@ -491,15 +358,17 @@ for(my $rule_index = 0 ; $rule_index < @$dependency_rules ; $rule_index++)
 # continue with single definition of dependencies 
 # and remove temporary dependency names
 #-------------------------------------------------------------------------
-my @dependencies = () ;
+my @dependencies ;
 for my $dependency_name (keys %$tree)
 	{
-	if(($dependency_name !~ /^__/) && ('' eq ref $tree->{$dependency_name}{NAME}))
-		{
-		push @dependencies, $tree->{$dependency_name}  ;
-		}
+	push @dependencies, $tree->{$dependency_name} unless($dependency_name =~ /^__/) ;
 	}
-
+	
+for (keys %$tree)
+	{
+	delete $tree->{$_} if $_ !~ /^__/ ;
+	}
+	
 #-------------------------------------------------------------------------
 # handle IGNORE_LOCAL_RULES, ...
 #-------------------------------------------------------------------------
@@ -507,11 +376,15 @@ if(@sub_pbs)
 	{
 	if(@sub_pbs == 1)
 		{
-		if(@dependencies && exists $sub_pbs[0]{SUBPBS}{IGNORE_LOCAL_RULES} && $sub_pbs[0]{SUBPBS}{IGNORE_LOCAL_RULES} > 0)
+		if(@dependencies && exists $sub_pbs[0]->{IGNORE_LOCAL_RULES} && $sub_pbs[0]->{IGNORE_LOCAL_RULES} > 0)
 			{
 			PrintWarning
 				(
-				DumpTree(\@sub_pbs, "Sub Pbs rule:")
+				DumpTree
+					(
+					  \@sub_pbs
+					, "Sub Pbs rule:"
+					)
 				) ;
 				
 			PrintWarning("Forces removal of locally defined dependencies:\n") ;
@@ -527,7 +400,6 @@ if(@sub_pbs)
 			for my $dependency (keys %$tree)
 				{
 				next if $dependency =~ /^__/ ;
-
 				delete $tree->{$dependency} ;
 				
 				unless(exists $inserted_nodes->{$dependency}{__LINKED})
@@ -674,23 +546,16 @@ for my $dependency (@dependencies)
 	if(exists $inserted_nodes->{$dependency_name})
 		{
 		# the dependency already exists within the tree (inserted through another node)
-		$tree->{$dependency_name} = $inserted_nodes->{$dependency_name} ;
-		$tree->{$dependency_name}{__LINKED}++ ;
+		# do some sanity checking
+		#~ PrintDebug DumpTree($inserted_nodes->{$dependency_name}, '' , MAX_DEPTH => 3) ;
 		
-		my $display_linked_node_info = 0 ;
-		$display_linked_node_info++ if($pbs_config->{DEBUG_DISPLAY_DEPENDENCIES} && (! $pbs_config->{NO_LINK_INFO})) ;
-		
-		my $rule_info =  $dependency_rules->[$rule_index]{NAME} . $dependency_rules->[$rule_index]{ORIGIN} ;
-							
-		my $linked_node_info = "      Linking '$dependency_name'" ;
-		$linked_node_info   .= " (from $inserted_nodes->{$dependency_name}{__INSERTED_AT}{INSERTION_FILE}" ;
-		$linked_node_info   .= " -> $inserted_nodes->{$dependency_name}{__INSERTED_AT}{INSERTION_RULE})\n" ;
-		
-		$linked_node_info   .= "         Above node is not depended yet!\n" unless (exists $inserted_nodes->{$dependency_name}{__DEPENDED}) ;
-			
 		if($inserted_nodes->{$dependency_name}{__INSERTED_AT}{INSERTION_FILE} ne $Pbsfile)
 			{
-			die ERROR("--no_external_link switch specified, stop.\n") if(defined $pbs_config->{DEBUG_NO_EXTERNAL_LINK}) ;
+			if(defined $pbs_config->{DEBUG_NO_EXTERNAL_LINK})
+				{
+				PrintError("--no_external_link switch specified, stop.\n") ;
+				die ;
+				}
 				
 			unless($pbs_config->{NO_LOCAL_MATCHING_RULES_INFO})
 				{
@@ -704,27 +569,49 @@ for my $dependency (@dependencies)
 				
 				if(exists $inserted_nodes->{$dependency_name}{__DEPENDED} && @local_rules_matching)
 					{
-					my $local_matching_rules_warning = '' ;
+					my $rule_info =  $dependency_rules->[$rule_index]{NAME}
+										. $dependency_rules->[$rule_index]{ORIGIN} ;
+										
+					PrintWarning
+						(
+						  "While checking '$node_name' at rule '$rule_info' :\n"
+						. "   Existing node '$dependency_name' from:"
+						. "$inserted_nodes->{$dependency_name}{__INSERTED_AT}{INSERTION_FILE}"
+						. "->$inserted_nodes->{$dependency_name}{__INSERTED_AT}{INSERTION_RULE}"
+						. " is already depended but has local rules matching in '$Pbsfile':\n"
+						) ;
 					
 					for my $matching_rule_index (@local_rules_matching)
 						{
-						$local_matching_rules_warning .=
-							"            $matching_rule_index:"
-							. $dependency_rules->[$matching_rule_index]{NAME}
-							. $dependency_rules->[$matching_rule_index]{ORIGIN}
-							. "\n" ;
+						$rule_info =  $dependency_rules->[$matching_rule_index]{NAME}
+											. $dependency_rules->[$matching_rule_index]{ORIGIN} ;
+											
+						PrintWarning("      $matching_rule_index:$rule_info\n") ;
 						}
 						
-					if($local_matching_rules_warning ne '')
-						{
-						$linked_node_info .= "         Ignoring local matching rules from '$Pbsfile':\n$local_matching_rules_warning" ;
-						$display_linked_node_info++ ;
-						}
 					}
 				}
 			}
 			
-		PrintWarning $linked_node_info if $display_linked_node_info ;
+		if($pbs_config->{DEBUG_DISPLAY_DEPENDENCIES} && (! $pbs_config->{NO_LINK_INFO}))
+			{
+			my $rule_info =  $dependency_rules->[$rule_index]{NAME}
+								. $dependency_rules->[$rule_index]{ORIGIN} ;
+								
+			PrintWarning
+				(
+				"In Pbsfile : $Pbsfile, while at rule $rule_info, node '$node_name'\n"
+				. "    '$dependency_name' already inserted @ $inserted_nodes->{$dependency_name}{__INSERTED_AT}{INSERTION_FILE}:"
+				. "$inserted_nodes->{$dependency_name}{__INSERTED_AT}{INSERTION_RULE}, Linking.\n"
+				) ;
+				
+			PrintWarning("    Above node is not depended yet!\n") unless (exists $inserted_nodes->{$dependency_name}{__DEPENDED}) ;
+			
+			#~ warn DumpTree($inserted_nodes->{$dependency_name}, "node '$dependency_name' linked but not depended:") ;
+			}
+		
+		$tree->{$dependency_name} = $inserted_nodes->{$dependency_name} ;
+		$tree->{$dependency_name}{__LINKED}++ ;
 		}
 	else
 		{
@@ -782,11 +669,171 @@ for my $dependency (@dependencies)
 		}
 	}
 	
-if($has_matching_non_subpbs_rules)
+if(0 == $has_dependencies)
+	{
+	if($pbs_config->{DEBUG_DISPLAY_DEPENDENCIES} && $node_name_matches_ddr)
+		{
+		if($node_name !~/^__/)
+			{
+			PrintInfo
+				(
+				"$node_name has no locally defined dependencies "
+				. "(rules from package '$package_alias' loaded from '"
+				. $pbs_config->{PBSFILE}
+				. "').\n"
+				) ;
+			}
+		}
+	
+	if(@sub_pbs)
+		{
+		if(@sub_pbs != 1)
+			{
+			PrintError "In Pbsfile : $Pbsfile, $node_name has multiple sub pbs defined:\n" ;
+			PrintError(DumpTree(\@sub_pbs,, "Sub Pbs:")) ;
+			
+			Carp::croak  ;
+			}
+			
+		# the node had no dependencie but a single subpbs matched
+		
+		my $sub_pbs_hash    = $sub_pbs[0] ;
+		my $sub_pbs_name    = $sub_pbs_hash->{PBSFILE} ;
+		my $sub_pbs_package = $sub_pbs_hash->{PACKAGE} ;
+		
+		my $alias_message = '' ;
+		$alias_message = "aliased as '$sub_pbs_hash->{ALIAS}'" if(defined $sub_pbs_hash->{ALIAS}) ;
+		
+		my $source_directories = $pbs_config->{SOURCE_DIRECTORIES} ;
+		my $sub_pbs_name_stem ;
+		
+		if(File::Spec->file_name_is_absolute($sub_pbs_name))
+			{
+			for my $source_directory (@$source_directories)
+				{
+				if($^O eq 'MSWin32' ? $sub_pbs_name =~ /^$source_directory(.*)/i :
+				   					  $sub_pbs_name =~ /^$source_directory(.*)/)
+					{
+					$sub_pbs_name_stem = $1 ;
+					last ;
+					}
+				}
+			}
+		else
+			{
+			$sub_pbs_name_stem = $sub_pbs_name ;
+			}
+		
+		if(! exists $sub_pbs_hash->{NON_RELOCATABLE_PBSFILE} && defined $sub_pbs_name_stem)
+			{
+			$sub_pbs_name_stem =~ s/^\.\/// ;
+			$sub_pbs_name_stem =~ s/\/// ;
+			
+			for my $source_directory (@$source_directories)
+				{
+				#~ PrintDebug "Trying: $source_directory/$sub_pbs_name_stem\n" ;
+				
+				if(-e "$source_directory/$sub_pbs_name_stem")
+					{
+					if("$source_directory/$sub_pbs_name_stem" ne $sub_pbs_name)
+						{
+						PrintWarning2("Relocated '$sub_pbs_name' @ '$source_directory/$sub_pbs_name_stem'\n") ;
+						$sub_pbs_name = "$source_directory/$sub_pbs_name_stem" ;
+						last ;
+						}
+					}
+				}
+			}
+			
+		unless(defined $pbs_config->{NO_SUBPBS_INFO})
+			{
+			if(defined $pbs_config->{SUBPBS_FILE_INFO})
+				{
+				PrintWarning("[$PBS::PBS::pbs_runs] Depending '$node_name' $alias_message with sub pbs '$sub_pbs_package:$sub_pbs_name'.\n") ;
+				}
+			else
+				{
+				PrintWarning("Depending '$node_name' $alias_message.\n") ;
+				}
+			}
+			
+		#-------------------------------------------------------------
+		# run subpbs
+		#-------------------------------------------------------------
+		
+		delete $inserted_nodes->{$node_name} ; # temporarely eliminate ourself from the existing nodes list
+		
+		my $tree_name = "sub_pbs$sub_pbs_name" ;
+		$tree_name =~ s/.\//_/g ;
+		
+		PrintInfo(DumpTree($sub_pbs_hash, "sub pbs:")) if defined $pbs_config->{DISPLAY_SUB_PBS_DEFINITION} ;
+			
+		# Synchonize with elements from the sub pbs definition, specially build and source dirs 
+		# we overide elements
+		my $sub_pbs_config = {%{$tree->{__PBS_CONFIG}}, %$sub_pbs_hash} ;
+		$sub_pbs_config->{PARENT_PACKAGE} = $package_alias ;
+		$sub_pbs_config->{PBS_COMMAND} ||= DEPEND_ONLY ;
+		
+		my $sub_node_name = $node_name ;
+		$sub_node_name    = $sub_pbs_hash->{ALIAS} if(defined $sub_pbs_hash->{ALIAS}) ;
+		
+		my $package_config = PBS::Config::GetPackageConfig($load_package) ;
+		my %sub_config = PBS::Config::ExtractConfig
+								(
+								$package_config
+								, $tree->{__PBS_CONFIG}{CONFIG_NAMESPACES}
+								, ['CURRENT', 'PARENT', 'COMMAND_LINE', 'PBS_FORCED'] # LOCAL REMOVED!
+								) ;
+								
+		#~ PrintError(DumpTree($sub_pbs_config, "subpbs config for package $sub_pbs_name :")) ;
+		
+		my $already_inserted_nodes = $inserted_nodes ;
+		$already_inserted_nodes    = {} if(defined $sub_pbs_hash->{LOCAL_NODES}) ;
+		
+		
+		my $sub_tree = PBS::PBS::Pbs
+						(
+						  $sub_pbs_name
+						, $load_package
+						, $sub_pbs_config
+						, \%sub_config
+						, [$sub_node_name]
+						, $already_inserted_nodes
+						, $tree_name
+						, $sub_pbs_config->{PBS_COMMAND}
+						) ;
+						
+		# keep this node insertion info
+		$sub_tree->{$sub_node_name}{__INSERTED_AT}{ORIGINAL_INSERTION_DATA} =  $tree->{__INSERTED_AT} ;
+		
+		# keep parent relationship
+		for my $dependency_to_key (keys %{$tree->{__DEPENDENCY_TO}})
+			{
+			$sub_tree->{$sub_node_name}{__DEPENDENCY_TO}{$dependency_to_key} = $tree->{__DEPENDENCY_TO}{$dependency_to_key};
+			}
+			
+		# copy the data generated by subpbs
+		for my $new_key (keys %{$sub_tree->{$sub_node_name}})
+			{
+			# keep some  attributes defined from the current Pbs
+			next if $new_key =~ /__NAME/ ;
+			next if $new_key =~ /__USER_ATTRIBUTE/ ;
+			next if $new_key =~ /__LINKED/ ;
+			
+			$tree->{$new_key} = $sub_tree->{$sub_node_name}{$new_key} ;
+			}
+			
+		# make ourself the real node again
+		$inserted_nodes->{$node_name} = $tree ;
+		
+		#~warn Data::Dumper->Dump($sub_tree, "sub_tree:\n") ;
+		}
+	}
+else
 	{
 	if(@sub_pbs)
 		{
-		PrintError "In Pbsfile : $Pbsfile, $node_name has locally matching rules and matching subpbs definition:\n" ;
+		PrintError "In Pbsfile : $Pbsfile, $node_name has locally defined dependencies and matching subpbs definition:\n" ;
 		for my $dependency (keys %$tree)
 			{
 			next if $dependency =~ /^__/ ;
@@ -798,12 +845,6 @@ if($has_matching_non_subpbs_rules)
 		Carp::croak ;
 		}
 		
-	# a node can be inserted from diffrent pbs file, still the result should be the same
-	# if the rules applied to the node are identical, we thus only remeber the pbsfile whith matching rules
-	$tree->{__DEPENDING_PBSFILE} = PBS::Digest::GetFileMD5($Pbsfile) ;
-	$tree->{__LOAD_PACKAGE} = $load_package;
-	#~ $tree->{__DEPENDING_PACKAGE} = $load_package;
-	
 	for my $dependency (keys %$tree)
 		{
 		next if $dependency =~ /^__/ ;
@@ -850,7 +891,7 @@ if($has_matching_non_subpbs_rules)
 					}
 				}
 				
-			PrintWarning("Ignoring local matching rules from '$Pbsfile':\n$ignored_rules") if $ignored_rules ne '' ;
+			PrintWarning("Local rules from '$Pbsfile' are ignored.\n$ignored_rules") if $ignored_rules ne '' ;
 			}
 			
 		unless(exists $tree->{$dependency}{__DEPENDED})
@@ -859,137 +900,8 @@ if($has_matching_non_subpbs_rules)
 			}
 		}
 	}
-else
-	{
-	if(@sub_pbs)
-		{
-		if(@sub_pbs != 1)
-			{
-			PrintError "In Pbsfile : $Pbsfile, $node_name has multiple sub pbs defined:\n" ;
-			PrintError(DumpTree(\@sub_pbs,, "Sub Pbs:")) ;
-			
-			Carp::croak  ;
-			}
-			
-		# the node had no dependencie but a single subpbs matched
-		
-		my $sub_pbs_hash    = $sub_pbs[0]{SUBPBS} ;
-		my $sub_pbs_name    = $sub_pbs_hash->{PBSFILE} ;
-		my $sub_pbs_package = $sub_pbs_hash->{PACKAGE} ;
-		
-		my $alias_message = '' ;
-		$alias_message = "aliased as '$sub_pbs_hash->{ALIAS}'" if(defined $sub_pbs_hash->{ALIAS}) ;
-		
-		$sub_pbs_name = LocatePbsfile($pbs_config, $Pbsfile, $sub_pbs_name, $sub_pbs[0]{RULE}) ;
-		$sub_pbs_hash->{PBSFILE_LOCATED} = $sub_pbs_name ;
-		
-		unless(defined $pbs_config->{NO_SUBPBS_INFO})
-			{
-			if(defined $pbs_config->{SUBPBS_FILE_INFO})
-				{
-				my $node_info = "inserted at '$inserted_nodes->{$node_name}->{__INSERTED_AT}{INSERTION_RULE}'" ;
-				PrintWarning("[$PBS::PBS::pbs_runs/$PBS::PBS::Pbs_call_depth] Depending '$node_name' $alias_message, $node_info, with sub pbs '$sub_pbs_package:$sub_pbs_name'.\n") ;
-				}
-			else
-				{
-				PrintWarning("Depending '$node_name' $alias_message.\n") ;
-				}
-			}
-			
-		#-------------------------------------------------------------
-		# run subpbs
-		#-------------------------------------------------------------
-		
-		delete $inserted_nodes->{$node_name} ; # temporarely eliminate ourself from the existing nodes list
-		
-		my $tree_name = "sub_pbs$sub_pbs_name" ;
-		$tree_name =~ s/.\//_/g ;
-		
-		PrintInfo(DumpTree($sub_pbs_hash, "sub pbs:")) if defined $pbs_config->{DISPLAY_SUB_PBS_DEFINITION} ;
-			
-		# Synchonize with elements from the sub pbs definition, specially build and source dirs 
-		# we overide elements
-		my $sub_pbs_config = {%{$tree->{__PBS_CONFIG}}, %$sub_pbs_hash} ;
-		$sub_pbs_config->{PARENT_PACKAGE} = $package_alias ;
-		$sub_pbs_config->{PBS_COMMAND} ||= DEPEND_ONLY ;
-		
-		my $sub_node_name = $node_name ;
-		$sub_node_name    = $sub_pbs_hash->{ALIAS} if(defined $sub_pbs_hash->{ALIAS}) ;
-		
-		my $package_config = PBS::Config::GetPackageConfig($load_package) ;
-		my %sub_config = PBS::Config::ExtractConfig
-								(
-								$package_config
-								, $tree->{__PBS_CONFIG}{CONFIG_NAMESPACES}
-								, ['CURRENT', 'PARENT', 'COMMAND_LINE', 'PBS_FORCED'] # LOCAL REMOVED!
-								) ;
-								
-		#~ PrintError(DumpTree($sub_pbs_config, "subpbs config for package $sub_pbs_name :")) ;
-		
-		my $already_inserted_nodes = $inserted_nodes ;
-		$already_inserted_nodes    = {} if(defined $sub_pbs_hash->{LOCAL_NODES}) ;
-		
-		my ($build_result, $build_message, $sub_tree, $inserted_nodes, $sub_pbs_load_package)
-			= PBS::PBS::Pbs
-				(
-				  $sub_pbs_name
-				, $load_package
-				, $sub_pbs_config
-				, \%sub_config
-				, [$sub_node_name]
-				, $already_inserted_nodes
-				, $tree_name
-				, $sub_pbs_config->{PBS_COMMAND}
-				) ;
-			
-		# keep this node insertion info
-		$sub_tree->{$sub_node_name}{__INSERTED_AT}{ORIGINAL_INSERTION_DATA} =  $tree->{__INSERTED_AT} ;
-		
-		# keep parent relationship
-		for my $dependency_to_key (keys %{$tree->{__DEPENDENCY_TO}})
-			{
-			$sub_tree->{$sub_node_name}{__DEPENDENCY_TO}{$dependency_to_key} = $tree->{__DEPENDENCY_TO}{$dependency_to_key};
-			}
-			
-		# copy the data generated by subpbs
-		for my $new_key (keys %{$sub_tree->{$sub_node_name}})
-			{
-			# keep some  attributes defined from the current Pbs
-			next if $new_key =~ /__NAME/ ;
-			next if $new_key =~ /__USER_ATTRIBUTE/ ;
-			next if $new_key =~ /__LINKED/ ;
-			
-			$tree->{$new_key} = $sub_tree->{$sub_node_name}{$new_key} ;
-			}
-			
-		# make ourself the real node again
-		$inserted_nodes->{$node_name} = $tree ;
-		
-		#~warn Data::Dumper->Dump($sub_tree, "sub_tree:\n") ;
-		}
-	else
-		{
-		if(0 == $has_dependencies && $pbs_config->{DEBUG_DISPLAY_DEPENDENCIES} && $node_name_matches_ddr)
-			{
-			next if $node_name =~ /^__/ ;
-			
-			my $dependency_info = '' ;
-			
-			if(@{$tree->{__MATCHING_RULES}})
-				{
-				$dependency_info = "   '$node_name' has no locally defined dependencies" ;
-				}
-			else
-				{
-				$dependency_info = "   '$node_name' wasn't depended" ;
-				}
-				
-			PrintInfo "$dependency_info (rules from '$pbs_config->{PBSFILE}').\n" ;
-			}
-		}
-	}
 	
-if($tree->{__IMMEDIATE_BUILD}  && ! exists $tree->{__BUILD_DONE})
+if($tree->{__IMMEDIATE_BUILD})
 	{
 	PrintInfo2("** Immediate build of node $node_name **\n") ;
 	my(@build_sequence, %trigged_files) ;
@@ -1006,8 +918,6 @@ if($tree->{__IMMEDIATE_BUILD}  && ! exists $tree->{__BUILD_DONE})
 		, \@build_sequence
 		, \%trigged_files
 		) ;
-		
-	RunPluginSubs($pbs_config, 'PostDependAndCheck', $pbs_config, $tree, $inserted_nodes, \@build_sequence, $tree) ;
 	
 	if($pbs_config->{DO_BUILD})
 		{
@@ -1020,7 +930,7 @@ if($tree->{__IMMEDIATE_BUILD}  && ! exists $tree->{__BUILD_DONE})
 			
 		if($build_result == BUILD_SUCCESS)
 			{
-			#~ PrintInfo2("** Immediate build of node '$node_name' Done **\n") ;
+			PrintInfo2("** Immediate build of node '$node_name' Done **\n") ;
 			}
 		else
 			{
@@ -1052,134 +962,6 @@ if($PBS::Debug::debug_enabled)
 
 #-------------------------------------------------------------------------------
 
-sub LocatePbsfile
-{
-my $pbs_config   = shift ;
-my $Pbsfile      = shift ;
-my $sub_pbs_name = shift ;
-my $rule         = shift ;
-
-my $info = $pbs_config->{ADD_ORIGIN} ? "rule '$rule->{NAME}' at '$rule->{FILE}\:$rule->{LINE}'" : '' ;
-
-my $source_directories = $pbs_config->{SOURCE_DIRECTORIES} ;
-my $sub_pbs_name_stem ;
-
-if(file_name_is_absolute($sub_pbs_name))
-	{
-	PrintWarning "Using absolute subpbs: '$sub_pbs_name' $info.\n" ;
-	}
-else
-	{
-	my ($basename, $path, $ext) = File::Basename::fileparse($Pbsfile, ('\..*')) ;			
-	
-	my $found_pbsfile ;
-	for my $source_directory (@$source_directories, $path)
-		{
-		my $searched_pbsfile = PBS::PBSConfig::CollapsePath("$source_directory/$sub_pbs_name") ;
-		
-		if(-e $searched_pbsfile)
-			{
-			if($found_pbsfile)
-				{
-				if($pbs_config->{DISPLAY_SUBPBS_SEARCH_INFO})
-					{
-					PrintInfo "Ignoring pbsfile '$sub_pbs_name' in '$source_directory' $info.\n" ;
-					}
-				}
-			else
-				{
-				if($pbs_config->{DISPLAY_SUBPBS_SEARCH_INFO})
-					{
-					PrintInfo "Located pbsfile '$sub_pbs_name' in '$source_directory' $info.\n" ;
-					}
-					
-				$found_pbsfile = $searched_pbsfile ;
-				
-				last unless $pbs_config->{DISPLAY_ALL_SUBPBS_ALTERNATIVES} ;
-				}
-			}
-		else
-			{
-			if($pbs_config->{DISPLAY_SUBPBS_SEARCH_INFO})
-				{
-				PrintInfo "Couldn't find pbsfile '$sub_pbs_name' in '$source_directory' $info.\n" ;
-				}
-			}
-		}
-		
-	my $sub_pbs_name_stem ;
-	$found_pbsfile ||= "$path$sub_pbs_name" ;
-	
-	#check if we can find it somewhere else in the source directories
-	for my $source_directory (@$source_directories)
-		{
-		my $flag = '' ;
-		$flag = '(?i)' if $^O eq 'MSWin32' ;
-		
-		if($found_pbsfile =~ /$flag^$source_directory(.*)/)
-			{
-			$sub_pbs_name_stem = $1
-			}
-		}
-		
-	my $relocated_subpbs ;
-	if(defined $sub_pbs_name_stem)
-		{
-		if($pbs_config->{DISPLAY_SUBPBS_SEARCH_INFO})
-			{
-			PrintInfo "Found stem '$sub_pbs_name_stem'.\n" ;
-			}
-			
-		for my $source_directory (@$source_directories)
-			{
-			my $relocated_from_stem = PBS::PBSConfig::CollapsePath("$source_directory/$sub_pbs_name_stem") ;
-			
-			if(-e $relocated_from_stem)
-				{
-				unless($relocated_subpbs)
-					{
-					$relocated_subpbs = $relocated_from_stem  ;
-					
-					if($relocated_from_stem ne $found_pbsfile)
-						{
-						PrintWarning2("Relocated '$sub_pbs_name_stem' in '$source_directory' $info.\n") ;
-						}
-					else
-						{
-						if($pbs_config->{DISPLAY_SUBPBS_SEARCH_INFO})
-							{
-							PrintInfo "Keeping '$sub_pbs_name_stem' from '$source_directory' $info.\n" ;
-							}
-						}
-						
-					last unless $pbs_config->{DISPLAY_ALL_SUBPBS_ALTERNATIVES} ;
-					}
-				else
-					{
-					if($pbs_config->{DISPLAY_SUBPBS_SEARCH_INFO})
-						{
-						PrintInfo "Ignoring relocation of '$sub_pbs_name_stem' in '$source_directory' $info.\n" ;
-						}
-					}
-				}
-			else
-				{
-				if($pbs_config->{DISPLAY_SUBPBS_SEARCH_INFO})
-					{
-					PrintInfo "Couldn't relocate '$sub_pbs_name_stem' in '$source_directory' $info.\n" ;
-					}
-				}
-			}
-		}
-		
-	$sub_pbs_name = $relocated_subpbs || $found_pbsfile || $sub_pbs_name;
-	}
-
-return($sub_pbs_name) ;
-}
-
-#-------------------------------------------------------------------------------------------------------
-
 1 ;
 
 __END__
