@@ -9,9 +9,9 @@ use warnings ;
 use Data::Dumper ;
 use Data::TreeDumper ;
 use Carp ;
+use Tie::IxHash ;
 use Tie::Hash::Indexed ;
 use Time::HiRes qw(gettimeofday tv_interval) ;
-use File::Spec::Functions qw(:ALL) ;
 
 require Exporter ;
 use AutoLoader qw(AUTOLOAD) ;
@@ -20,7 +20,7 @@ our @ISA = qw(Exporter) ;
 our %EXPORT_TAGS = ('all' => [ qw() ]) ;
 our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } ) ;
 our @EXPORT = qw(PbsUse) ;
-our $VERSION = '0.03' ;
+our $VERSION = '0.02' ;
 
 use PBS::PBSConfig ;
 use PBS::Output ;
@@ -36,21 +36,6 @@ use PBS::Digest;
 
 use Digest::MD5 qw(md5_hex) ;
 
-#-------------------------------------------------------------------------------
-
-# a global place to keep timing and other pbs run information
-# the idea is to make them available to a post pbs script for processing
-# this should of course be passed around not be global, maybe we 
-# should package this and the dependency tree, nodes, etc in some structure
-
-our $pbs_run_information = 
-	{
-	# TIMING => {}
-	# CAHE => {MD5_HITS => xxx, C_DEPENDER_HITS => YYY ...
-	# BUILDER
-	} ;
-
-
 #-------------------------------------------------------------------------------
 
 our $Pbs_call_depth = -1 ;
@@ -72,7 +57,7 @@ my $Pbsfile              = shift ;
 my $parent_package       = shift ;
 my $pbs_config           = shift ;
 my $parent_config        = shift ;
-my $package              = CanonizePackageName($pbs_config->{PACKAGE}) ;
+my $package              = $pbs_config->{PACKAGE} ;
 my $build_directory      = $pbs_config->{BUILD_DIRECTORY} ;
 my $source_directories   = $pbs_config->{SOURCE_DIRECTORIES} ;
 my $targets              = shift ;
@@ -87,15 +72,10 @@ unless('' eq ref $package && '' ne $package)
 	die ;
 	}
 
-if(defined $pbs_config->{SAVE_CONFIG})
-	{
-	SaveConfig($targets, $Pbsfile, $pbs_config, $parent_config) ;
-	}
-
 undef $pbs_config->{TARGETS} ;
 for my $target (@$targets)
 	{
-	if(file_name_is_absolute($target) || $target =~ /^\.\//)
+	if(File::Spec->file_name_is_absolute($target) || $target =~ /^\.\//)
 		{
 		push @{$pbs_config->{TARGETS}}, $target ;
 		}
@@ -137,7 +117,7 @@ for (@{$pbs_config->{DISPLAY_PBS_CONFIGURATION}})
 
 if($display_all_pbs_config)
 	{
-	PrintInfo DumpTree($pbs_config, "Package '$package:$Pbsfile' config:") ;
+	warn DumpTree($pbs_config, "Package '$package:$Pbsfile' config:") ;
 	}
 else
 	{
@@ -189,7 +169,6 @@ tie my %tree_hash, "Tie::Hash::Indexed" ;
 
 my $dependency_tree = \%tree_hash ;
 my $build_point = '' ;
-my ($build_result, $build_message) ;
 
 if(-e $Pbsfile || defined $pbs_config->{PBSFILE_CONTENT})
 	{
@@ -232,7 +211,9 @@ if(-e $Pbsfile || defined $pbs_config->{PBSFILE_CONTENT})
 	push my @config_namespaces, @{$pbs_config->{CONFIG_NAMESPACES}} ;
 	
 	my $user_build ;
-	my $sub_config ;	
+	my $sub_config ;	PBS::PBSConfig::RegisterPbsConfig($load_package, $pbs_config) ;
+
+	
 	PBS::PBSConfig::RegisterPbsConfig($load_package, $pbs_config) ;
 	
 	#Command defines
@@ -275,7 +256,6 @@ if(-e $Pbsfile || defined $pbs_config->{PBSFILE_CONTENT})
 		  . "use PBS::Check ;\n"
 		  . "use PBS::PBS ;\n"
 		  . "use PBS::Digest;\n"
-		  . "use PBS::Rules::Creator;\n"
 		  . $add_pbsfile_digest
 		  
 		, "\n# load OK\n1 ;\n"
@@ -297,7 +277,7 @@ if(-e $Pbsfile || defined $pbs_config->{PBSFILE_CONTENT})
 				{
 				my @targets = map 
 							{
-							if(file_name_is_absolute($_) || /^\.\//)
+							if(File::Spec->file_name_is_absolute($_) || /^\.\//)
 								{
 								"$_" ;
 								}
@@ -329,23 +309,22 @@ if(-e $Pbsfile || defined $pbs_config->{PBSFILE_CONTENT})
 		{
 		PrintInfo("Using user defined Build().\n") unless $pbs_config->{DISPLAY_NO_STEP_HEADER} ;
 		
-		($build_result, $build_message)
-			= $user_build->
-				(
-				  $Pbsfile
-				, $package
-				, $load_package
-				, $pbs_config
-				, \@rule_namespaces
-				, $rules
-				, \@config_namespaces
-				, $sub_config
-				, $targets # automatically build in rule 'BuiltIn::__ROOT', given as information only
-				, $inserted_nodes
-				, $dependency_tree # rule 0 dependent name is in $dependency_tree ->{__NAME}
-				, $build_point
-				, $depend_and_build
-				) ;
+		$user_build->
+			(
+			  $Pbsfile
+			, $package
+			, $load_package
+			, $pbs_config
+			, \@rule_namespaces
+			, $rules
+			, \@config_namespaces
+			, $sub_config
+			, $targets # automatically build in rule 'BuiltIn::__ROOT', given as information only
+			, $inserted_nodes
+			, $dependency_tree # rule 0 dependent name is in $dependency_tree ->{__NAME}
+			, $build_point
+			, $depend_and_build
+			) ;
 			
 		}
 	else
@@ -360,27 +339,25 @@ if(-e $Pbsfile || defined $pbs_config->{PBSFILE_CONTENT})
 			
 		if($pbs_config->{DISPLAY_COMPACT_DEPEND_INFORMATION})
 			{
-			my $number_of_nodes = scalar(keys %$inserted_nodes) ;
-			PrintInfo("PBS depend run $pbs_runs at depth: $Pbs_call_depth [$number_of_nodes].        \r", 0) ;
+			PrintInfo("PBS depend run $pbs_runs at depth: $Pbs_call_depth.       \r", 0) ;
 			}
 		
-		($build_result, $build_message)
-			= PBS::DefaultBuild::DefaultBuild
-				(
-				  $Pbsfile
-				, $package
-				, $load_package
-				, $pbs_config
-				, \@rule_namespaces
-				, $rules
-				, \@config_namespaces
-				, $sub_config
-				, $targets # automatically build in rule 'BuiltIn::__ROOT', given as information only
-				, $inserted_nodes
-				, $dependency_tree
-				, $build_point
-				, $depend_and_build
-				) ;
+		PBS::DefaultBuild::DefaultBuild
+									(
+									  $Pbsfile
+									, $package
+									, $load_package
+									, $pbs_config
+									, \@rule_namespaces
+									, $rules
+									, \@config_namespaces
+									, $sub_config
+									, $targets # automatically build in rule 'BuiltIn::__ROOT', given as information only
+									, $inserted_nodes
+									, $dependency_tree
+									, $build_point
+									, $depend_and_build
+									) ;
 									
 		}
 	}
@@ -397,60 +374,7 @@ if($pbs_config->{DISPLAY_DEPENDENCY_TIME})
 	PrintInfo(sprintf("Time in Pbsfile: %0.2f s.\n", tv_interval ($t0, [gettimeofday]))) ;
 	}
 	
-return($build_result, $build_message, $dependency_tree, $inserted_nodes, $load_package) ;
-}
-
-#-------------------------------------------------------------------------------
-
-sub SaveConfig
-{
-my ($targets, $pbsfile, $pbs_config, $parent_config) = @_ ;
-
-my $first_target = $targets->[0] ;
-my ($first_target_name, $first_target_path, $sufix) = File::Basename::fileparse($targets->[0], ('\..*')) ;
-$first_target_name .= $sufix ;
-
-(my $pbsfile_canonized = $pbsfile) =~ s/[^a-zA-Z0-9]/_/g ;
-my $path             = $pbs_config->{BUILD_DIRECTORY} . '/' . $first_target_path ;
-
-my $config_file_name = $path . 'config_' . $pbsfile_canonized . '___' . $first_target_name . '_' . $pbs_config->{SAVE_CONFIG} . '.pl' ;
-$config_file_name =~ s/[^a-zA-Z0-9\/.]/_/g ;
-
-use File::Path ;
-mkpath($path) unless(-e $path) ;
-
-PrintDebug "Saving Config in $config_file_name\n" ;
-
-open(CONFIG, ">", $config_file_name) or die qq[Can't open '$config_file_name': $!] ;
-
-local $Data::Dumper::Purity = 1 ;
-local $Data::Dumper::Indent = 1 ;
-local $Data::Dumper::Sortkeys = undef ;
-
-local $SIG{'__WARN__'} = sub 
-	{
-	if($_[0] =~ 'Encountered CODE ref')
-		{
-		# ignore this warning
-		}
-	else
-		{
-		print STDERR $_[0] ;
-		}
-	} ;
-
-print CONFIG PBS::Log::GetHeader('Config', $pbs_config) ;
-print CONFIG <<EOI ;
-# pbsfile: $pbsfile
-# target: $first_target
-
-EOI
-print CONFIG Data::Dumper->Dump([$parent_config], ['config']) ;
-print CONFIG Data::Dumper->Dump([$pbs_config], ['pbs_config']) ;
-
-print CONFIG 'return($pbs_config, $config);';
-
-close(CONFIG) ;
+return($dependency_tree, $inserted_nodes) ;
 }
 
 #-------------------------------------------------------------------------------
@@ -468,11 +392,6 @@ for my $source_name (@{[@_]})
 		die  ERROR("PbsUse only accepts strings as input. Called @ $file_name:$line.\n")  ;
 		}
 		
-	unless(defined $source_name)
-		{
-		die  ERROR("PbsUse must be given a name. Called @ $file_name:$line.\n")  ;
-		}
-		
 	my $t0 = [gettimeofday];
 	
 	my $global_package_dependency = shift || 1 ; # if set, the use module becomes adependency for all the package nodes
@@ -488,21 +407,14 @@ for my $source_name (@{[@_]})
 		die ;
 		}
 	
-	if(file_name_is_absolute($source_name))
+	for my $lib_path (@{$pbs_config->{LIB_PATH}})
 		{
-		$located_source_name = $source_name ;
-		}
-	else
-		{
-		for my $lib_path (@{$pbs_config->{LIB_PATH}})
+		$lib_path .= '/' unless $lib_path =~ /\/$/ ;
+		
+		if(-e $lib_path . $source_name)
 			{
-			$lib_path .= '/' unless $lib_path =~ /\/$/ ;
-			
-			if(-e $lib_path . $source_name)
-				{
-				$located_source_name = $lib_path . $source_name ;
-				last ;
-				}
+			$located_source_name = $lib_path . $source_name ;
+			last ;
 			}
 		}
 	
@@ -516,9 +428,7 @@ for my $source_name (@{[@_]})
 	$pbs_use_level++ ; # indent the PbsUse output to make the hierachy more visible
 	my $indentation = '   ' x $pbs_use_level ;
 	
-	PrintInfo("${indentation}PbsUse: '$located_source_name' called at '$file_name:$line'\n") if(defined $pbs_config->{DISPLAY_PBSUSE_VERBOSE}) ;
-	PrintInfo("${indentation}PbsUse: '$source_name'\n") if(defined $pbs_config->{DISPLAY_PBSUSE}) ;
-	
+	PrintInfo("${indentation}PbsUse: '$located_source_name'\n") if(defined $pbs_config->{DISPLAY_PBSUSE}) ;
 	
 	if(exists $files_loaded_via_PbsUse{$package}{$located_source_name})
 		{
@@ -532,7 +442,7 @@ for my $source_name (@{[@_]})
 		
 		if($global_package_dependency)
 			{
-			$add_as_package_dependency = "PBS::Digest::AddPbsLibDependencies('$located_source_name', '$source_name') ;\n" ;
+			$add_as_package_dependency = "PBS::Digest::AddPbsLibDependencies('$located_source_name:$source_name') ;\n" ;
 			}
 			
 		LoadFileInPackage
@@ -583,19 +493,12 @@ return DumpTree($files_loaded_via_PbsUse{__STATISTIC}, "'PbsUse' statistic:", DI
 }
 
 #-------------------------------------------------------------------------------
-sub CanonizePackageName
-{
-my $package = shift || die ;
-$package =~ s/[^a-zA-Z0-9_:]+/_/g ;
-
-return($package) ;
-}
 
 sub LoadFileInPackage
 {
 my $type       = shift ;
 my $file       = shift ;
-my $package    = CanonizePackageName(shift) ;
+my $package    = shift ;
 my $pbs_config = shift ;
 my $pre_code   = shift || '' ;
 my $post_code  = shift || '' ;
@@ -604,7 +507,7 @@ my $file_body = '' ; #?? can't let this variable undef or perl bugs out!
 
 if($type eq 'Pbsfile')
 	{
-	PrintInfo("==>Loading file '$file' into package '$package'.\n") if (defined $pbs_config->{DISPLAY_PBSFILE_LOADING}) ;
+	PrintInfo("==>Loading file '$file' into package '$package'.\n") if (defined $pbs_config->{DISPLAY_PACKAGE_LOADING}) ;
 	
 	if(defined $pbs_config->{PBSFILE_CONTENT} && -e $file)
 		{
